<?php
/**
 * Pakkepost Woocommerce Plugin
 *
 * @package           pakkepost-woocommerce
 * @author            Kodenor AS
 * @copyright         2023 Kodenor AS
 * @license           GPL-2.0-or-later
 *
 * @wordpress-plugin
 * Plugin Name:       Pakkepost Woocommerce Plugin
 * Plugin URI:        https://www.pakkepost.no/info/nettbutikk-moduler/worldpress-woocommerce-plugin
 * Description:       WooCommerce Pakkepost Shipping Method and live rates
 * Version:           3.1.1
 * Requires at least: 5.2
 * Requires PHP:      7.2
 * Author:            Kodenor AS
 * Author URI:        https://www.kodenor.no
 * Text Domain:       plugin-slug
 * License:           GPL v2 or later
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */

if (!defined('ABSPATH')) {
    exit;
}

if (!defined('WPINC')) {
    die('security by preventing any direct access to your plugin file');
}

// Declare that this plugin supports WooCommerce HPOS.
add_action( 'before_woocommerce_init', function() {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
	\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
} );

require_once __DIR__ . '/includes/class-frontend.php';
require_once __DIR__ . '/includes/class-metabox.php';

add_filter('https_local_ssl_verify', '__return_true');

define("WF_BRING_ID", "Pakkepost");

function getValue($key)
{
    $ini_array = parse_ini_file("plugin.ini");
     $value =  $ini_array[$key];
    return $value;
}

/**
 * Check if WooCommerce is active
 */
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    /**
     * WC_Pakkepost class
     */
    class Pakkepost_WooCommerce_Shipping
    {

        /**
         * Constructor
         */
        public function __construct()
        {

            add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'wf_plugin_action_links'));
            add_action('wp_ajax_my_action', array($this, 'my_action_pakkepost'));
            add_action('wp_ajax_my_action_pakkepost', array($this, 'my_action_pakkepost'));
            add_action('wp_ajax_nopriv_my_action', array($this, 'my_action_pakkepost'));
            add_action('wp_ajax_nopriv_my_action_pakkepost', array($this, 'my_action_pakkepost'));
            add_filter('woocommerce_get_sections_shipping', array($this, 'pakkepost_shipping_settings_tab'));
            add_filter('woocommerce_get_settings_shipping', array($this, 'pakkepost_shipping_settings'), 10, 2);
            add_action('manage_posts_extra_tablenav', array($this, 'add_extra_button'));

            $frontend = new PakkepostFrontend();
            add_action( 'woocommerce_after_shipping_rate', array($frontend,'shipping_product_custom_select_field'), 20, 2 );
            add_action( 'woocommerce_after_shipping_rate', array($frontend,'packages_select_field'), 20, 2 );
            add_action( 'wp_footer', array($frontend,'shipping_product_script_js') );
            add_action( 'wp_footer', array($frontend,'packages_script_js') );
            add_action( 'wp_ajax_product_name', array($frontend,'set_shipping_product_name') );
            add_action( 'wp_ajax_nopriv_product_name', array($frontend,'set_shipping_product_name') );
            add_action( 'wp_ajax_packages', array($frontend,'set_packages_number') );
            add_action( 'wp_ajax_nopriv_packages', array($frontend,'set_packages_number') );
            add_action( 'wp_ajax_save_service_location', array($frontend,'handle_save_service_location'));
            add_action( 'wp_ajax_nopriv_save_service_location', array($frontend,'handle_save_service_location')); // For logged-out users
            add_action( 'woocommerce_checkout_process', array($frontend,'shipping_product_checkout_validation')); 
            add_action( 'woocommerce_checkout_create_order', array($frontend,'save_shipping_product_as_order_meta'), 30, 1 );
            add_action( 'woocommerce_checkout_create_order', array($frontend,'save_packages_as_order_meta'), 30, 1 );
            add_action( 'woocommerce_checkout_create_order', array($frontend,'save_service_location_as_order_meta'), 10, 2);

            $adminMetabox = new AdminMetabox();
            add_action('add_meta_boxes', array($adminMetabox,'add_meta_boxesws'));
            add_action( 'add_meta_boxes', array($adminMetabox, 'admin_order_custom_metabox'));
            add_action( 'woocommerce_admin_order_data_after_shipping_address', array($adminMetabox,'admin_order_display_pakkepost'), 30, 1 );
            add_filter( 'woocommerce_get_order_item_totals', array($adminMetabox,'display_shipping_product_on_order_item_totals', 50, 3 ));
            add_filter( 'woocommerce_get_order_item_totals', array($adminMetabox,'display_packages_on_order_item_totals', 50, 3 ));

        }


        public function add_extra_button($where)
        {

            if ('top' === $where) {
                echo ' <a href="' . (admin_url('admin-ajax.php?action=my_action_pakkepost')) . '" id="book_shipment" class="button" onclick="return bookShipment(this, event, false);" >Book Shipment</a>';
            }
        }

        public function pakkepost_shipping_settings($settings, $current_section)
        {

            $custom_settings = array();

            if ('pakkepost' == $current_section) {

                $custom_settings = array(

                    array(
                        'name' => __('Pakkepost'),
                        'type' => 'title',
                        'desc' => __('Pakkepost version streamlines your complete shipping process and saves time'),
                        'id' => 'pakkepost',
                    ),

                    array(
                        'name' => __('Pakkepost User Id'),
                        'type' => 'text',
                        'id' => 'pakkepost_user_id',
                        'custom_attributes' => array(
                            'required' => 'required'
                        ),
                    ),

                    array(
                        'name' => __('Pakkepost Subscription Id'),
                        'type' => 'text',
                        'id' => 'pakkepost_subscription_id',
                        'custom_attributes' => array(
                            'required' => 'required'
                        ),
                    ),
                    array(
                        'name' => __('Shop Name'),
                        'type' => 'text',
                        'id' => 'pakkepost_shop_name',
                        'custom_attributes' => array(
                            'required' => 'required'
                        ),
                    ),

                    array(
                        'name' => __('Pakkepost API Mode'),
                        'type' => 'select',
                        'desc' => __('Set as Test to switch to Pakkepost api test servers. Transaction will be treated as sample transactions by Pakkepost.'),
                        'desc_tip' => true,
                        'id' => 'api_mode',
                        'options' => array(

                            'Live' => __('Live'),
                            'Test' => __('Test'),
                        ),
                    ),
                    array(
                        'name' => __('Supplier'),
                        'type' => 'select',
                        'desc' => __('Select your supplier.'),
                        'desc_tip' => true,
                        'id' => 'pakkepost_supplier',
                        'options' => array(
                            'Bring' => __('Bring'),
                            'Postnord' => __('Postnord'),
                        ),

                    ),
                    array(
                        'name' => __('Sender Email'),
                        'type' => 'text',
                        'id' => 'sender_email',
                        'custom_attributes' => array(
                            'required' => 'required'
                        ),
                    ),
                    array(
                        'name' => __('Sender Phone Number'),
                        'type' => 'text',
                        'id' => 'sender_phone_number',
                        'custom_attributes' => array(
                            'required' => 'required'
                        ),
                    ),
                    array(
                        'type' => 'sectionend',
                        'id' => 'pakkepost_shipping'
                    ),
                );

                return $custom_settings;
            } else {
                return $settings;
            }

        }

        public function pakkepost_shipping_settings_tab($settings_tab)
        {
            $settings_tab['pakkepost'] = __('Pakkepost');
            return $settings_tab;
        }

        public function my_action_pakkepost()
        {
            global $wpdb;
            $order_ids = array();
            foreach ($_REQUEST["shipping_type"] as $orderId => $shipping_type) {
                $order_ids[] = $orderId;
            }
            $supplier = (get_option('pakkepost_supplier'));

            $api_mode = (get_option('api_mode') != '') ? get_option('api_mode') : 'Test';
            $pakkepost_user_id = (get_option('pakkepost_user_id') != '') ? get_option('pakkepost_user_id') : '863';

            $pakkepost_shop_name = (get_option('pakkepost_shop_name') != '') ? get_option('pakkepost_shop_name') : 'Shop 1';
            $pakkepost_subscription_id = (get_option('pakkepost_subscription_id') != '') ? get_option('pakkepost_subscription_id') : 'Test';
            $sender_email = (get_option('sender_email') != '') ? get_option('sender_email') : '';
            $sender_phone_number = (get_option('sender_phone_number') != '') ? get_option('sender_phone_number') : '';

            $store_address = get_option('woocommerce_store_address');
            $store_address_2 = get_option('woocommerce_store_address_2');
            $store_city = get_option('woocommerce_store_city');
            $store_postcode = get_option('woocommerce_store_postcode');

            $headers = array(
                "cache-control: no-cache",
                "content-type: application/json",
            );
            if ("Live" == $api_mode) {
                $headers[] = 'Ocp-Apim-Subscription-Key: ' . $pakkepost_subscription_id;
            }

            $data = array();

            $numberOfPackages = $_REQUEST["number_of_packages"];
            if ($numberOfPackages === "undefined") {
                $numberOfPackages = "1";
            }
            foreach ($order_ids as $key => $orderId) {
                $order = wc_get_order($orderId);
                $user = $order->get_user();
                $shipping_address = $order->get_address('shipping');
                if (empty($shipping_address['city']) && (empty($shipping_address['postcode']))) {
                    $shipping_address = $order->get_address('billing');
                }
                $shipping_type = (int)$_REQUEST["shipping_type"][$orderId];
                if (!empty($_REQUEST["home_delivery"][$orderId])) {
                    $additionalServices = 'hjemlevering';
                } else {
                    $additionalServices = '';
                }

                $data[] = array(
                    "userId" => (int)$pakkepost_user_id,
                    "senderName" => $pakkepost_shop_name,
                    "senderAddressLine" => $store_address,
                    "senderAddressLineNumber" => $store_address_2,
                    "senderPostalCode" => $store_postcode,
                    "senderCity" => $store_city,
                    "senderReference" => "WP" . $orderId ,
                    "senderEmail" => $sender_email,
                    "senderPhoneNumber" => (int)$sender_phone_number,
                    "recipientName" => $shipping_address["first_name"] . " " . $shipping_address["last_name"],
                    "recipientAddressLine" => $shipping_address["address_1"],
                    "recipientAddressLineNumber" => $shipping_address["address_2"],
                    "recipientPostalCode" => $shipping_address["postcode"],
                    "recipientCity" => $shipping_address["city"],
                    "recipientReference" => "",
                    "recipientPhoneNumber" => $order->get_billing_phone(),
                    "recipientEmail" => $order->get_billing_email(),
                    "productId" => (int)$shipping_type,
                    "additionalServices" => $additionalServices,
                    "numberOfPackages" => $numberOfPackages
                );
            }
            if (empty($data)) throw new Exception('No data found');
            if ("Live" == $api_mode) {
                $url = getValue('PAKKEPOST_API_PROD_URL');
            } else {
                $url = getValue('PAKKEPOST_API_TEST_URL');
            }
            $curl = curl_init($url);
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_VERBOSE, 1);
            //curl_setopt($curl, CURLOPT_STDERR, $fp);
            curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            $response = curl_exec($curl);
            $err = curl_error($curl);
            $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            $jsonResponse = json_decode($response);
            if ($err) {
                echo "cURL Error #:" . $err;
            } else {

                $order->update_meta_data( '_pakkepost_order_id', $jsonResponse->orderId );
                $order->save();

                if ($jsonResponse->payment_link !== NULL) {
                    if (filter_var($jsonResponse->payment_link, FILTER_VALIDATE_URL) === false) {
                        echo $jsonResponse->payment_link;
                    } else {
                        header("Location: " . $jsonResponse->payment_link);
                        die();
                    }
                } else {
                    echo "<center> Det oppstod en feil: <br /> <br />" ."<b>". $jsonResponse->message[0] ."</b>";
                    echo "<br /> <br /> Vennligst prøv igjen senere eller kontakt support.";

                    wp_die();
                }
            }
            wp_die();
        }
        /**
         * Plugin page links
         */
        public function wf_plugin_action_links($links)
        {
            $plugin_links = array(
                '<a href="' . admin_url(getValue('ADMIN_URL')) . '">' . __('Settings', 'pakkepost_shipping') . '</a>',
            );
            return array_merge($plugin_links, $links);
        }
    }

    new Pakkepost_WooCommerce_Shipping();
    
}