<?php

/**
 * Class PakkepostFrontend
 *
 * This class handles the frontend functionality for the Pakkepost WooCommerce plugin.
 */
class PakkepostFrontend
{

    // Add carriers
    // Custom function that handle your settings
    public function carrier_settings()
    {

        switch (get_option('pakkepost_supplier')) {
            case 'Bring':
                $products = array(
                    // The option displayed at first ( or keep an empty value '',)
                    __("Velg pakkestørrelse", "woocommerce"),

                    // Bring
                    'Norge S - 35x22x10cm 1kg',
                    'Norge M - 10kg',
                    'Norge L - 20kg',
                    'Norge XL - 35kg',
                    'Bedrift S - 10kg',
                    'Bedrift M - 35kg'
                );
                break;
            case 'Postnord':
                $products = array(
                    // The option displayed at first ( or keep an empty value '',)
                    __("Velg pakkestørrelse", "woocommerce"),

                    // Postnord
                    'Pakkeautomat - 50x30x20cm 5kg',
                    'Medium - 5kg',
                    'Stor - 10kg',
                    'XL - 20kg',
                    'XXL - 35 kg',
                    'Bedrift Liten - 5kg',
                    'Bedrift Medium - 15kg',
                    'Bedrift Medium - 15kg'
                );
                break;
        }

        return array(
            'targeted_methods' => array('pakkepost'), // Your targeted shipping method(s) in this array
            'field_id'         => 'product_name', // Field Id
            'field_type'       => 'select', // Field type
            'field_label'      => '', // Leave empty value if the first option has a text (see below).
            'label_name'       => __("Shipping product", "woocommerce"), // for validation and as meta key for orders
            'field_options'    => $products,
        );
    }
    public function packages_settings()
    {

        return array(
            'targeted_methods' => array('pakkepost'), // Your targeted shipping method(s) in this array
            'package_id'         => 'packages', // Field Id
            'package_type'       => 'select', // Field type
            'package_label'      => '', // Leave empty value if the first option has a text (see below).
            'package_label_name'       => __("Pakkepost", "woocommerce"), // for validation and as meta key for orders
            'package_options'    => array('Velg antall pakker', '1', '2', '3', '4', '5'),
        );
    }

    function shipping_product_custom_select_field($method, $index)
    {
        extract($this->carrier_settings());

        $value   = WC()->session->get($field_id);
        $value   = WC()->session->__isset($field_id) ? $value : WC()->checkout->get_value('_' . $field_id);

        $options = array();

        echo '<div class="custom-carrier">';

        foreach ($field_options as $key => $option_value) {
            $option_key = $key == 0 ? '' : $key;
            $options[$option_key] = $option_value;
        }

        woocommerce_form_field('product_name', array( // Unique field ID for carrier name
            'type'     => $field_type,
            'label'    => '',
            'class'    => array('form-row-wide ' . 'product_name' . '-' . $field_type),
            'required' => true,
            'options'  => $options,
            'default' => 'Empty | Velg pakkestørrelse'
        ), $value);

        echo '</div>';
    }

    function packages_select_field($method, $index)
    {
        extract($this->packages_settings());

        $packages_value   = WC()->session->__isset($package_id) ? $value : WC()->checkout->get_value('_' . $package_id);

        $options = array();

        echo '<div class="packages">';

        // Loop through number of packages to add the correct keys
        foreach ($package_options as $key => $option_value1) {
            $option_key = $key == 0 ? '' : $key;
            $options[$option_key] = $option_value1;
        }

        woocommerce_form_field('packages', array( // Unique field ID for packages
            'type'     => $package_type,
            'label'    => '', // Not required if the first option has a text.
            'class'    => array('form-row-wide ' . 'packages' . '-' . $package_type),
            'required' => true,
            'options'  => $options,
        ), $packages_value);

        echo '</div>';
        
        echo '<div id="serviceLocations"> </div>';

        $js_variable = is_cart() ? 'wc_cart_params' : 'wc_checkout_params';

        ?>
        <script type="text/javascript">
            jQuery(function($) {
                
                $('#serviceLocations').on('change', 'select#serviceLocations', function() {
                    var selectedLocation = $(this).val();
                    if (typeof <?php echo $js_variable; ?> === 'undefined') {
                        console.error('Required WooCommerce parameters not found');
                        return false;
}
                    $.ajax({
                        type: 'POST',
                        url: <?php echo $js_variable; ?>.ajax_url,
                        data: {
                            'action': 'save_service_location',
                            'location': selectedLocation
                        },
                        
                        success: function(response) {
                            console.log('Location saved:', response); // Log the response (for debugging)
                    },
                        error: function(error) {
                            console.error('Error saving location:', error); // Log errors
                        }
                    });
                });
            });
        </script>
        <?php

    }



    function shipping_product_script_js()
    {
        // Only cart & checkout pages
        if (is_cart() || (is_checkout() && !is_wc_endpoint_url())) :

            // Load settings and convert them in variables
            extract($this->carrier_settings());

            $js_variable = is_cart() ? 'wc_cart_params' : 'wc_checkout_params';

            // jQuery Ajax code
?>
            <script type="text/javascript">
                jQuery(function($) {
                    if (typeof <?php echo $js_variable; ?> === 'undefined')
                        return false;

                    let pakkepost_subscription_id = '<?php echo esc_js(get_option('pakkepost_subscription_id')); ?>';

                    $(document.body).on('change', 'select#<?php echo $field_id; ?>', function() {
                        var value = $(this).val();

                        let consignee_postal_code = jQuery("#shipping_postcode").val() || jQuery("#billing_postcode").val();

                        $.ajax({
                            type: 'POST',
                            url: <?php echo $js_variable; ?>.ajax_url,
                            data: {
                                'action': 'product_name',
                                'value': value
                            },
                            success: function(result) {

                                <?php if (WP_DEBUG) : ?>
                                   console.log(result);
                                <?php endif; ?>
                                if (result.includes('Pakkeautomat')) {

                                    let store_postal_code = <?php echo get_option('woocommerce_store_postcode'); ?>;

                                    var settings = {
                                        "url": "https://pakkepost.azure-api.net/locations",
                                        "method": "POST",
                                        "timeout": 0,
                                        "headers": {
                                            "Content-Type": "application/json",
                                            "Ocp-Apim-Subscription-Key": pakkepost_subscription_id
                                        },
                                        "body": JSON.stringify({
                                            "consignorZip": store_postal_code,
                                            "consigneeZip": consignee_postal_code
                                        })
                                    };
                                    fetch(settings.url, settings)
                                        .then(response => response.json())
                                        .then(data => {
                                            // Process the API response data (data) here
                                            populateSelect(data); // Call a function to populate the select list
                                        })
                                        .catch(error => {
                                            console.error(error);
                                        });

                                    function populateSelect(data) {
                                        // Clear existing options (optional)
                                        $("#serviceLocations select").empty();
                                        $("#serviceLocations select").append('<option value="">Velg utleveringssted</option>')
                                        // Loop through the API response data
                                        data.forEach(location => {
                                            // Assuming 'location' has a property 'name' for the option text
                                            var option = $('<option></option>').val(location.name).text(location.name);
                                            $("#serviceLocations select").append(option);
                                        });
                                    }

                                    // Initial select list with a default option (optional)
                                    $("#serviceLocations").html(
                                        '<div id="serviceLocations">' +
                                        '  <select id="serviceLocations" class="form-row-wide serviceLocations-select">' +
                                        '    <option value="">Velg utleveringssted</option>' + // Default option
                                        '  </select>' +
                                        '</div>'
                                    );
                                }
                            }
                        });
                    });

                });
            </script>
        <?php
        endif;
    }


    function packages_script_js()
    {
        // Only cart & checkout pages
        if (is_cart() || (is_checkout() && !is_wc_endpoint_url())) :

            // Load settings and convert them in variables
            extract($this->packages_settings());

            $js_variable = is_cart() ? 'wc_cart_params' : 'wc_checkout_params';

            // jQuery Ajax code
        ?>
            <script type="text/javascript">
                jQuery(function($) {
                    if (typeof <?php echo $js_variable; ?> === 'undefined')
                        return false;

                    $(document.body).on('change', 'select#<?php echo $package_id; ?>', function() {
                        var value = $(this).val();
                        $.ajax({
                            type: 'POST',
                            url: <?php echo $js_variable; ?>.ajax_url,
                            data: {
                                'action': 'packages',
                                'packages_value': value
                            },
                            success: function(result) {
                                <?php if (WP_DEBUG) : ?>
                                    console.log(result);
                                <?php endif; ?>                            
                            }
                        });

                    });

                });
            </script>
<?php
        endif;
    }



    function set_shipping_product_name()
    {
        if (isset($_POST['value'])) {
            // Load settings and convert them in variables
            extract($this->carrier_settings());
            if (empty($_POST['value'])) {
                $value = 0;
                $label = 'Empty';
            } else {
                $value = $label = esc_attr($_POST['value']);
            }


            // Update session variable
            WC()->session->set($field_id, $value);

            // Send back the data to javascript (json encoded)
            echo $label . ' | ' . $field_options[$value];

        }
    }

    function handle_save_service_location() {

        if (isset($_POST['location'])) {

            $selectedLocation = sanitize_text_field($_POST['location']); // Sanitize the input
    
            // Save the selected location in the session
            WC()->session->set('service_location', $selectedLocation);
    
            // Optionally, return a response
            wp_send_json_success('Location saved: ' . $selectedLocation);
        } else {
            wp_send_json_error('No location provided');
        }
    
        wp_die(); // Terminate the execution of the script
    }

    function set_packages_number()
    {
        if (isset($_POST['packages_value'])) {
            // Load settings and convert them in variables
            extract($this->packages_settings());

            if (empty($_POST['packages_value'])) {
                $packages_value = 0;
                $label = 'Empty';
            } else {
                $packages_value = $label = esc_attr($_POST['packages_value']);
            }

            // Update session variable
            WC()->session->set($package_id, $packages_value);

            // Send back the data to javascript (json encoded)
            echo $label . ' | ' . $package_options[$packages_value];
            die();
        }
    }



    function shipping_product_checkout_validation()
    {

        // Load settings and convert them in variables
        extract($this->carrier_settings());

        if (isset($_POST[$field_id]) && empty($_POST[$field_id])) {
            wc_add_notice(
                sprintf(
                    __("Please select a %s as it is a required field.", "woocommerce"),
                    '<strong>' . $label_name . '</strong>'
                ),
                "error"
            );
        }
    }

    public function save_shipping_product_as_order_meta($order)
    {

        // Load settings and convert them in variables
        extract($this->carrier_settings());

        if (isset($_POST[$field_id]) && !empty($_POST[$field_id])) {
            $order->update_meta_data('_' . $field_id, $field_options[esc_attr($_POST[$field_id])]);
            WC()->session->__unset($field_id); // remove session variable
        }
    }

    public function save_packages_as_order_meta($order)
    {
        // Load settings and convert them in variables
        extract($this->packages_settings());

        if (isset($_POST[$package_id]) && !empty($_POST[$package_id])) {
            $order->update_meta_data('_' . $package_id, $package_options[esc_attr($_POST[$package_id])]);
            WC()->session->__unset($package_id); // remove session variable
        }
    }

    public function save_service_location_as_order_meta($order) {

        // Check if the service location is set in the session
        if (WC()->session->__isset('service_location')) {
            $serviceLocation = WC()->session->get('service_location');
            // Save the service location as order meta
            $order->update_meta_data('_service_location', $serviceLocation);
        }
    }


    public function display_shipping_product_on_order_item_totals($total_rows, $order, $tax_display)
    {

        $this->writeLog("display_shipping_product_on_order_item_totals");

        // Load settings and convert them in variables
        extract($this->carrier_settings());

        $carrier = $order->get_meta('_' . $field_id); // Get Shipping product

        if (!empty($carrier)) {
            $new_total_rows = [];

            // Loop through order total rows
            foreach ($total_rows as $key => $values) {
                $new_total_rows[$key] = $values;

                // Inserting the Shipping product under shipping method
                if ($key === 'shipping') {
                    $new_total_rows[$field_id] = array(
                        'label' => $label_name,
                        'value' => $carrier,
                    );
                }
            }
            return $new_total_rows;
        }
        return $total_rows;
    }

    public function display_packages_on_order_item_totals($total_rows, $order, $tax_display)
    {
        // Load settings and convert them in variables
        extract($this->packages_settings());

        $package = $order->get_meta('_' . $package_id); // Get packages

        if (!empty($package)) {
            $new_total_rows = [];

            // Loop through order total rows
            foreach ($total_rows as $key => $values) {
                $new_total_rows[$key] = $values;

                // Inserting the Shipping product under shipping method
                if ($key === 'shipping') {
                    $new_total_rows[$field_id] = array(
                        'label' => $label_name,
                        'value' => $carrier,
                    );
                }
            }
            return $new_total_rows;
        }
        return $total_rows;
    }

    function writeLog($message)
    {
        $logger = wc_get_logger();
        $logger->debug($message, array('source' => 'Pakkepost Plugin'));
    }
}
